package es.rotxo.escaner;

import android.content.Context;
import android.hardware.Camera;
import android.util.AttributeSet;
import android.util.Log;
import android.widget.SeekBar;

import org.opencv.android.JavaCameraView;

import java.util.List;

/**
 * Created by Rotxo on 08/07/2016.
 */
public class MiCamaraView extends JavaCameraView {

    Aplicacion aplic;
    protected SeekBar seekBar;

    //contrucctores
    public MiCamaraView(Context context, AttributeSet attrs) {

        super(context, attrs);
        aplic = (Aplicacion) context.getApplicationContext();
    }
    public MiCamaraView(Context context, int cameraId) {

        super(context, cameraId);
        aplic = (Aplicacion) context.getApplicationContext();
    }


    //inicializamos la camara con los valores que nos interesan
    protected boolean initializeCamera(int width, int height)
    {

        boolean ret = super.initializeCamera(width, height);

        //comprobamos la resolucion
        Camera.Size resolucionActal=getResolution();
        Log.e("Resolucion", "width:"+resolucionActal.width+" height: "+resolucionActal.height);
        List<Camera.Size> resoluciones=getResolutionList();
        if (resoluciones!=null)
        {
            if ((resolucionActal.width < resoluciones.get(0).width) || (resolucionActal.height < resoluciones.get(0).height)) {
                setResolution(resoluciones.get(0));
            }
        }


        Camera.Parameters params = mCamera.getParameters();



        //modificamos los modos de autofoco
        List<String> FocusModes = params.getSupportedFocusModes();
        if (FocusModes != null && FocusModes.contains(Camera.Parameters.FOCUS_MODE_CONTINUOUS_PICTURE)) {
            params.setFocusMode(Camera.Parameters.FOCUS_MODE_CONTINUOUS_PICTURE);
        }else
        {
            if (FocusModes != null && FocusModes.contains(Camera.Parameters.FOCUS_MODE_AUTO))
                params.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO);
        }
        //miramos si tiene la escena barcode
        List<String> ScenesModes =params.getSupportedSceneModes();

          if (ScenesModes != null && ScenesModes.contains(Camera.Parameters.SCENE_MODE_BARCODE))
          {
             params.setSceneMode(Camera.Parameters.SCENE_MODE_BARCODE);
          }
        //luz del móvil
        if (aplic.isTorch())
        {
            List<String> FlashModes = params.getSupportedFlashModes();
            if (FlashModes!=null) {
                if (FlashModes.contains(Camera.Parameters.FLASH_MODE_TORCH))
                    params.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);
            }else
            {
                aplic.setTorch(false);
            }
        }

        //ZOOM
        if(params.isZoomSupported()) {


            //activamos los controles zoom
            enableZoomControls(params);
            //ajustamos el zoom por defecto
            int zoom=damiZoom(params);
            if (zoom!=0)
            {
                params.setZoom(zoom);
            }
        }

        mCamera.setParameters(params);

        return ret;
    }
    public List<Camera.Size> getResolutionList() {
        return  mCamera.getParameters().getSupportedPreviewSizes();
    }

    public Camera.Size getResolution() {
        Camera.Parameters params = mCamera.getParameters();
        Camera.Size s = params.getPreviewSize();
        return s;
    }

    public void setResolution(Camera.Size resolution) {
        disconnectCamera();
        connectCamera((int)resolution.width, (int)resolution.height);
    }
    //zoom
    public void setZoomControl(SeekBar _seekBar)
    {
        seekBar=_seekBar;
    }
    protected void enableZoomControls(Camera.Parameters params)
    {

        final int maxZoom = params.getMaxZoom();
        seekBar.setMax(maxZoom);
        seekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener()
        {
            int progressvalue=0;
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser)
            {
                progressvalue=progress;
                Camera.Parameters params = mCamera.getParameters();
                params.setZoom(progress);
                mCamera.setParameters(params);

            }
            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {}
            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {}

        });

    }
    //Calcula el zoom en % segun preferencias
    private int damiZoom(Camera.Parameters params)
    {
        int zoom=0;
        int mizoom=aplic.getZoomValor();
        if ((mizoom>0)&&(mizoom<=100)) {
            int maxZoom = params.getMaxZoom();
            if (maxZoom>0)
                 zoom = (maxZoom * mizoom) / 100;
        }

        return zoom;
    }


}

