package es.rotxo.escaner;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Environment;
import android.os.Message;
import android.provider.ContactsContract;
import android.util.Log;
import android.widget.Toast;

import org.opencv.core.Core;
import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.core.MatOfFloat;
import org.opencv.core.MatOfInt;
import org.opencv.core.Point;
import org.opencv.core.Range;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;
import org.opencv.core.Size;
import org.opencv.imgcodecs.Imgcodecs;
import org.opencv.imgproc.Imgproc;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

/**
 * Created by Rotxo on 27/06/2016.
 */
public class Procesador {

    Mat binaria;
    Mat linea;
    Mat salida;
    Barras barras;
    Aplicacion aplic;







    //Constructor
    public Procesador(Context contexto)
    {
         binaria=new Mat();
         linea=new Mat();
         aplic = (Aplicacion) contexto.getApplicationContext();



    }



    public Mat procesa(Mat entrada) {



        salida = entrada.clone();
        //************************************Linea 1***********************************************
        // *****************************************************************************************
        linea=Linea1(entrada,salida);
        //*****************************PREPROCESO***************************************************
        SuvaizadoGausiano(linea,linea);

        ResizeLinea(linea,6000);
        //si esta la antorcha ecualizamos el histograma
        if (aplic.isTorch()) {
            Imgproc.equalizeHist(linea, linea);
        }

        //***************************** Binarizacion************************************************

           Binarizacion(linea,binaria);

        //SEGEMENTACION
        barras=Segmentacion(binaria);
        //DECODIFICACION
        barras.DeterminarTipo();
        if (barras.isEncontrado()) {

            if (barras.getTipo().equals("EAN13")) {
                Log.e("Barras", "Codigo Correcto " + barras.getEan13().getCodigo());

                aplic.setCodigoBarras(barras.getEan13().getCodigo());
                Log.e("Lectura", "Linea1");

                //guardamos los datos
                aplic.AñadirCodBarras(barras.getEan13().getCodigo(),"EAN13");


            }
            if (barras.getTipo().equals("EAN8")) {
                Log.e("Barras", "Codigo Correcto " + barras.getEan8().getCodigo());
                aplic.setCodigoBarras(barras.getEan8().getCodigo());
                Log.e("Lectura", "Linea1");

                //guardamos lo datos
                aplic.AñadirCodBarras(barras.getEan8().getCodigo(),"EAN8");
            }


        }



        return salida;
    }




    //***********************************************************************************************************************************************************************************************
  //**************************************************************** LINEAS HA ANALIZAR ***********************************************************************************************************

    //Calcula una linea para ver el codigo de barras

    public Mat Linea1(Mat entrada,Mat salida)
    {
        //numero de filas
        int numeroFilas= entrada.rows();
        int numeroColumnas=entrada.cols();
        int primeraLinea = numeroFilas / 2;

        final Point P1 = new Point(0, primeraLinea);
        final Point P2 = new Point(numeroColumnas, primeraLinea);
        Imgproc.line(salida, P1, P2, new Scalar(255, 255, 255));


        return entrada.row(primeraLinea);
    }
    //Dibujamos la linea de lectura
    public void DibujamosLinea(Mat entrada)
    {
        //numero de filas
        int numeroFilas= entrada.rows();
        int numeroColumnas=entrada.cols();
        int primeraLinea = numeroFilas / 2;

        //usamos fotos de testeo

        final Point P1 = new Point(0, primeraLinea);
        final Point P2 = new Point(numeroColumnas, primeraLinea);
        Imgproc.line(entrada, P1, P2, new Scalar(255, 255, 255));


    }



    //*******************************************************************************************************************************************************************************************
    //*************************************************************     PREPROCESO ******************************************************************************************************************
    //*******************************************************************************************************************************************************************************************
    //Aumentamos la imagen
    private void ResizeLinea(Mat linea,int x)
    {
     // res = cv2.resize(img,None,fx=2, fy=2, interpolation = cv2.INTER_CUBIC
        //imgproc.resize(Mat src,Mat dst,Size dsize,double fx,double fy,int interpolation)
        //INTER_CUBIC  It is a bi-cubic interpolation over 4x4 pixel neighbourhood.
        Imgproc.resize(linea, linea, new Size(x, 1), 0, 0,Imgproc.INTER_CUBIC);
    }
    //Suavizado Gausiano
    private void SuvaizadoGausiano(Mat entrada,Mat salida)
    {
       // src - input image; the image can have any number of channels, which are processed independently, but the depth should be CV_8U, CV_16U, CV_16S, CV_32F or CV_64F.
       // dst - output image of the same size and type as src.
        //    ksize - Gaussian kernel size. ksize.width and ksize.height can differ but they both must be positive and odd. Or, they can be zero's and then they are computed from sigma*.
       // sigmaX - Gaussian kernel standard deviation in X direction.

        int x= entrada.cols()-1;
        x=x/15;
        if ((x%2==1)&&(x>0)) {
            org.opencv.core.Size s = new Size(x, 1);
            Imgproc.GaussianBlur(entrada, salida, s, 0.8);
        }
    }

    //*******************************************************************************************************************************************************************************************
    //*****************************************************     BINARIZACION ********************************************************************************************************************
    //*******************************************************************************************************************************************************************************************


    public void Binarizacion(Mat entrada,Mat salida)
    {
        //umbral diferente en cada posición

        int x=entrada.cols()-1;
        x=x/15;
        Mat SE = Imgproc.getStructuringElement(Imgproc.MORPH_RECT, new Size(x,1));
        Mat gray_dilation= new Mat();
        Mat gray_erosion=new Mat();
        Mat umbral=new Mat();

        Imgproc.dilate(entrada, gray_dilation, SE);
        Imgproc.erode(entrada, gray_erosion, SE );
        Core.addWeighted(gray_dilation, 0.5,gray_erosion, 0.5, 0.0, umbral);//esto es lo mismo -->dst = src1*alpha + src2*beta + gamma;

        Mat mibinaria=new Mat();

        Core.subtract(umbral,entrada,mibinaria);
        //Procesador.guardarImagen(mibinaria, "Test", "binarizada");
        Imgproc.threshold(mibinaria, salida,1,255,Imgproc.THRESH_BINARY_INV);
        //Procesador.guardarImagen(salida, "Test", "binarizada");



        gray_dilation.release();
        gray_erosion.release();
        entrada.release();
        umbral.release();
        mibinaria.release();

        SE.release();

    }


    //*****************************************************************************************************************************************************************************************
    //**************************************************    SEGMENTACION **********************************************************************************************************************
    //*****************************************************************************************************************************************************************************************
    private Barras Segmentacion (Mat binaria)
    {
        int numeroColumnas=binaria.cols()-1;
        Barras b= new Barras();
        double[] pixel;
        Elemento d = new Elemento();
        int tipo=2;
        int posicionPantalla=0;
        for (int i=0;i<=numeroColumnas;i++)
        {
            //obtenemos el valor del pixel en el canal 0
            pixel=binaria.get(0,i);
            tipo=DaTipo(pixel[0]); //obtnemos si es 0 o 1

            //Si es la primera vez ponemos el tipo si es barra negra 1  barra blanca 0
            if (i==0)
            {
                d.setTipo(tipo);
                d.setInicio(0);
            }else
            {

               if (tipo==d.getTipo())
               {
                   // si es el mismo digito
                   d.setNpixels();//aumentaos el numero de digitos
               }else  //es un nuevo digito
               {
                   //Guardamos el anterior

                    b.GuardarDigito(d);
                   //creamos un nuevo digito
                   d=new Elemento(tipo);
                   d.setInicio(i);
               }
            }

        }
        return b;
    }
    //*******************************************************************************************************************************************************************************************
    //**************************************************************        VARIOS **************************************************************************************************************
    //*******************************************************************************************************************************************************************************************
    //Obtiene el valor del pixel si es cero o uno
    private int DaTipo(double digito)
    {
        int tipo=2;
        if (digito==255)
            tipo=0;
        if (digito==0)
            tipo=1;
        return tipo;
    }






}
